/*
 * This file is part of ShowMIDI.
 * Copyright (command) 2023 Uwyn LLC.  https://www.uwyn.com
 *
 * ShowMIDI is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ShowMIDI is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#pragma once

#include <JuceHeader.h>

#include "PluginSettings.h"

namespace showmidi
{
    class ShowMIDIPluginAudioProcessor  : public AudioProcessor
    {
    public:
        ShowMIDIPluginAudioProcessor();
        ~ShowMIDIPluginAudioProcessor() override;
        
        void prepareToPlay(double sampleRate, int samplesPerBlock) override;
        void releaseResources() override;
        
        void processBlock(AudioBuffer<float>&, MidiBuffer&) override;
        
        AudioProcessorEditor* createEditor() override;
        bool hasEditor() const override;
        
        const String getName() const override;
        
        bool acceptsMidi() const override;
        bool producesMidi() const override;
        bool isMidiEffect() const override;
        double getTailLengthSeconds() const override;
        
        int getNumPrograms() override;
        int getCurrentProgram() override;
        void setCurrentProgram(int index) override;
        const String getProgramName(int index) override;
        void changeProgramName(int index, const String& newName) override;
        
        void getStateInformation(MemoryBlock& destData) override;
        void setStateInformation(const void* data, int sizeInBytes) override;
        
        PluginSettings& getSettings();
        
    private:
        PluginSettings settings_;
        
        JUCE_DECLARE_NON_COPYABLE_WITH_LEAK_DETECTOR (ShowMIDIPluginAudioProcessor)
    };
}
